<?php

namespace Core\Http;

class Request
{
    /**
     * Get request method
     *
     * @return mixed
     */
    public function method(): mixed
    {
        return $_SERVER['REQUEST_METHOD'];
    }

    /**
     * Get request url
     *
     * @return string
     */
    public function url(): string
    {
        return strtok($_SERVER['REQUEST_URI'], '?');
    }

    /**
     * Check if request is specific method
     *
     * @param string $method
     * @return bool
     */
    public function is(string $method): bool
    {
        return ($this->method() === strtoupper($method));
    }

    /**
     * Check if request has POST data
     *
     * @param string $param
     * @return bool
     */
    public final function has(string $param): bool
    {
        return isset($_POST[$param]);
    }

    /**
     * Get POST data
     *
     * @param string|null $param
     * @param mixed|null  $default
     * @return mixed
     */
    public final function get(string|null $param = null, mixed $default = null): mixed
    {
        if($param == null) {
            return $_POST;
        }

        if($this->has($param)) {
            return $_POST[$param];
        }

        return $default;
    }

}